<?php

namespace ShineSoftware\Constriv\Controller\Hosted;

use Magento\Framework\App\Action\Context;
use Magento\Framework\App\ObjectManager;
use Magento\Framework\View\LayoutFactory;
use Magento\Sales\Model\Order;
use Magento\Sales\Model\OrderFactory;
use Magento\Quote\Model\QuoteFactory;
use Psr\Log\LoggerInterface;
use STATE\PENDING\PAYMENT;
use ShineSoftware\Constriv\Model\Gateways\HostedFactory;

class Redirect extends AbstractHosted {

    /**
     * @var HostedFactory
     */
    protected $_gatewaysHostedFactory;

    /**
     * @var OrderFactory
     */
    protected $_modelOrderFactory;

    /**
     * @var LayoutFactory
     */
    protected $_viewLayoutFactory;

    public function __construct(Context $context, HostedFactory $gatewaysHostedFactory, OrderFactory $modelOrderFactory, QuoteFactory $modelQuoteFactory, LayoutFactory $viewLayoutFactory, LoggerInterface $loggerInterface) {
        $this->_gatewaysHostedFactory = $gatewaysHostedFactory;
        $this->_modelOrderFactory = $modelOrderFactory;
        $this->_viewLayoutFactory = $viewLayoutFactory;

        parent::__construct($context, $modelQuoteFactory, $loggerInterface);
    }

    public function execute() {
        $constriv = $this->_gatewaysHostedFactory->create();
        $order = $this->_modelOrderFactory->create();
        $order->loadByIncrementId(ObjectManager::getInstance()->get('Magento\Checkout\Model\Session')->getLastRealOrderId());
        $this->loadLayout();
        $block = $this->_viewLayoutFactory->create()->createBlock('Magento\Framework\View\Element\Template', 'constriv', ['template' => 'constriv/hosted/redirect.phtml']);
        try {
            $hostedpageurl = $constriv->init($order);
            if ($hostedpageurl instanceof Exception) {
                ObjectManager::getInstance()->get('Magento\Framework\Model\Session')->addError($hostedpageurl->getMessage());
                $order->setState(Order::STATE_CANCELED, true, $hostedpageurl->getMessage());
                $order->save();
                $this->_redirect('checkout/cart');
                return false;
            }
            $this->_logger->debug(__METHOD__ . " (" . __LINE__ . ") - Redirect the user at : $hostedpageurl", null, 'shinesoftware_constriv.log');
            $block->assign('url', $hostedpageurl);
            $order->setState(Order::PAYMENT, true, "Redirecting to constriv hosted secure page...");
            $order->save();
        } catch (\Exception $e) {
            $this->_logger->debug(__METHOD__ . " (" . __LINE__ . ") - The user has been alerted by the message: " . $e->getMessage(), null, 'shinesoftware_constriv.log');
            ObjectManager::getInstance()->get('Magento\Framework\Model\Session')->addError($e->getMessage());
            $this->_redirect("checkout/cart/");
        }
        $this->_viewLayoutFactory->create()->getBlock('content')->append($block);
        $this->renderLayout();
    }

}
