<?php

namespace ShineSoftware\Constriv\Controller\Hosted;

use Magento\Checkout\Helper\Data as HelperData;
use Magento\Framework\App\Action\Context;
use Magento\Sales\Model\OrderFactory;
use Magento\Quote\Model\QuoteFactory;
use Psr\Log\LoggerInterface;
use ShineSoftware\Constriv\Model\Gateways\HostedFactory;

class Response extends AbstractHosted {

    /**
     * @var HostedFactory
     */
    protected $_gatewaysHostedFactory;

    /**
     * @var LoggerInterface
     */
    protected $_logLoggerInterface;

    /**
     * @var OrderFactory
     */
    protected $_modelOrderFactory;
    
    /**
     * @var QuoteFactory
     */
    protected $_modelQuoteFactory;

    /**
     * @var HelperData
     */
    protected $_helperData;

    public function __construct(Context $context, QuoteFactory $modelQuoteFactory, HostedFactory $gatewaysHostedFactory, LoggerInterface $logLoggerInterface, OrderFactory $modelOrderFactory, HelperData $helperData) {
        $this->_gatewaysHostedFactory = $gatewaysHostedFactory;
        $this->_logLoggerInterface = $logLoggerInterface;
        $this->_modelOrderFactory = $modelOrderFactory;
        $this->_helperData = $helperData;
        $this->_modelQuoteFactory = $modelQuoteFactory;

        parent::__construct($context, $modelQuoteFactory, $logLoggerInterface);
    }

    public function execute() {
        try {
            $constriv = $this->_gatewaysHostedFactory->create();
            $paymentid = $this->getRequest()->getParam('paymentid');
            $trackid = $this->getRequest()->getParam('trackid');
            $responsecode = $this->getRequest()->getParam('responsecode');
            $data = $this->getRequest()->getParams();
            ob_clean();
            if (empty($trackid)) {
                $this->_logLoggerInterface->debug(__METHOD__ . " (" . __LINE__ . ") - TrackID is empty! or you are trying to call the bank response page manually!", null, 'shinesoftware_constriv.log');
                $redirect = "REDIRECT=" . $this->_url->getBaseUrl() . 'constriv/hosted/error';
                $this->_logLoggerInterface->debug(__METHOD__ . " (" . __LINE__ . ") - Magento replies with this redirect path: $redirect", null, 'shinesoftware_constriv.log');
                die($redirect);
            }
            $this->_logLoggerInterface->debug(__METHOD__ . " (" . __LINE__ . ") - the bank server replies with these parameters: " . json_encode($this->getRequest()->getParams()), null, 'shinesoftware_constriv.log');
            if (!empty($responsecode) && $responsecode != "00") {
                $redirect = "REDIRECT=" . $this->_url->getBaseUrl() . 'constriv/hosted/error';
                $this->_logLoggerInterface->debug(__METHOD__ . " (" . __LINE__ . ") - Magento replies with this redirect path: $redirect", null, 'shinesoftware_constriv.log');
                die($redirect);
            }
            $order = $this->_modelOrderFactory->create()->loadByIncrementId($trackid);
            if (!$order) {
                $this->_logLoggerInterface->debug(__METHOD__ . " (" . __LINE__ . ") - Order has been not found!", null, 'shinesoftware_constriv.log');
                $redirect = "REDIRECT=" . $this->_url->getBaseUrl() . 'constriv/hosted/error';
                $this->_logLoggerInterface->debug(__METHOD__ . " (" . __LINE__ . ") - Magento replies with this redirect path: $redirect", null, 'shinesoftware_constriv.log');
                die($redirect);
            }
            if (!empty($data['result']) && ($data['result'] == "AUTHORISED" || $data['result'] == "CAPTURED" || $data['result'] == "APPROVED")) {
                $constriv->completeOrder($order, $data);
                $redirect = "REDIRECT=" . $this->_url->getBaseUrl() . 'constriv/hosted/landing';
                $redirect .= "?PaymentID=" . $paymentid;
                $this->_logLoggerInterface->debug(__METHOD__ . " (" . __LINE__ . ") - Magento replies with this redirect path: $redirect", null, 'shinesoftware_constriv.log');
                die($redirect);
            } else {
                $this->_helperData->sendPaymentFailedEmail($order, "Error Code: " . $this->getRequest()->getParam('result'));
                $redirect = "REDIRECT=" . $this->_url->getBaseUrl() . 'constriv/hosted/failure';
                $this->_logLoggerInterface->debug(__METHOD__ . " (" . __LINE__ . ") - Magento replies with this redirect path: $redirect", null, 'shinesoftware_constriv.log');
                die($redirect);
            }
        } catch (\Exception $e) {
            $this->_logLoggerInterface->debug("Magento error: " . $e->getMessage(), null, 'shinesoftware_constriv.log');
            $redirect = "REDIRECT=" . $this->_url->getBaseUrl() . 'constriv/hosted/error';
            $this->_logLoggerInterface->debug(__METHOD__ . " (" . __LINE__ . ") - Magento replies with this redirect path: $redirect", null, 'shinesoftware_constriv.log');
            die($redirect);
        }
        die();
    }

}
