<?php

namespace ShineSoftware\Constriv\Controller\Main;

use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Framework\App\ObjectManager;
use Magento\Quote\Model\QuoteFactory;
use Psr\Log\LoggerInterface;

abstract class AbstractMain extends Action {

    /**
     * @var QuoteFactory
     */
    protected $_modelQuoteFactory;

    /**
     * @var LoggerInterface
     */
    protected $_logLoggerInterface;

    public function __construct(Context $context, QuoteFactory $modelQuoteFactory, LoggerInterface $logLoggerInterface) {
        $this->_modelQuoteFactory = $modelQuoteFactory;
        $this->_logLoggerInterface = $logLoggerInterface;

        parent::__construct($context);
    }

    protected function _clearQuote() {
        $session = ObjectManager::getInstance()->get('Magento\Checkout\Model\Session');
        $quote = $this->_modelQuoteFactory->create()->load($this->_getApiQuoteId());
        $quote->setIsActive(false)->setReservedOrderId(NULL)->save();
        $session->replaceQuote($quote);
        $this->_logLoggerInterface->debug(__METHOD__ . " (" . __LINE__ . ") ---> The quote has been cleared!", Null, 'shinesoftware_constriv.log');
        return true;
    }

    protected function _getApiQuoteId() {
        $quoteId = ObjectManager::getInstance()->get('Magento\Checkout\Model\Session')->getData('apiQuoteId');
        return $quoteId;
    }

    protected function _getApiOrderId() {
        $orderId = ObjectManager::getInstance()->get('Magento\Checkout\Model\Session')->getData('apiOrderId');
        return $orderId;
    }

    protected function _checkSecurityToken($order, $securityToken) {
        $initSecurityToken = $order->getPayment()->getAdditionalInformation('securitytoken');
        $this->_logLoggerInterface->debug(__METHOD__ . " (" . __LINE__ . ") - Init Security token ($initSecurityToken = $securityToken)!", null, 'shinesoftware_constriv.log');
        return ($initSecurityToken == $securityToken) ? true : false;
    }

    public function getCheckout() {
        return ObjectManager::getInstance()->get('Magento\Checkout\Model\Session');
    }

}

;
