<?php

namespace ShineSoftware\Constriv\Model;

use Magento\Framework\Api\AttributeValueFactory;
use Magento\Framework\Api\ExtensionAttributesFactory;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\DataObject;
use Magento\Framework\Data\Collection\AbstractDb;
use Magento\Framework\Model\Context;
use Magento\Framework\Model\ResourceModel\AbstractResource;
use Magento\Framework\Registry;
use Magento\Payment\Helper\Data as HelperData;
use Magento\Payment\Model\Method\AbstractMethod;
use Magento\Payment\Model\Method\Logger;
use Magento\Sales\Model\Order;
use Magento\Sales\Model\Order\Payment\Transaction;
use Magento\Store\Model\ScopeInterface;
//use PAYMENT\HOSTED\GATEWAY;

class Gateway extends AbstractMethod {

    public function __construct(Context $context, Registry $registry, ExtensionAttributesFactory $extensionFactory, AttributeValueFactory $customAttributeFactory, HelperData $paymentData, ScopeConfigInterface $scopeConfig, Logger $logger, AbstractResource $resource = null, AbstractDb $resourceCollection = null, array $data = []) {

        parent::__construct($context, $registry, $extensionFactory, $customAttributeFactory, $paymentData, $scopeConfig, $logger, $resource, $resourceCollection, $data);
    }

    const GATEWAY = 'hosted';

    protected function getOperationType($info) {
        switch ($info['operation_type']) {
            case "hcc":
                return Mage::getUrl('constriv/hosted');
            default:
                return Mage::getUrl('constriv/hosted');
        }
    }

    public function assignData(\Magento\Framework\DataObject $data) {
        if (!($data instanceof DataObject)) {
            $data = new DataObject($data);
        }
        $info = $this->getInfoInstance();
        $info->setCcType($data->getCcType())
                ->setMethod($data->getMethod())
                ->setCcOwner($data->getCcOwner())
                ->setCcLast4(substr($data->getCcNumber(), -4))
                ->setCcNumber($data->getCcNumber())
                ->setCcCid($data->getCcCid())
                ->setCcExpMonth($data->getCcExpMonth())
                ->setCcExpYear($data->getCcExpYear())
                ->setCcSsIssue($data->getCcSsIssue())
                ->setCcSsStartMonth($data->getCcSsStartMonth())
                ->setCcSsStartYear($data->getCcSsStartYear());
        $info->setAdditionalInformation('constriv', $data->toArray());
        return $this;
    }

    public function setAdditionalInformation($order, $transactionType = Transaction::TYPE_AUTH, array $additionalInfo) {
        $payment = $order->getPayment();
        if ($payment) {
            $transaction = $payment->lookupTransaction(false, $transactionType);
            $transaction->setAdditionalInformation(Transaction::RAW_DETAILS, $additionalInfo);
            $transaction->save();
            return true;
        }
        return false;
    }

    public function completeOrder($order, $additional_data) {
        if (!empty($order) && $order->getId()) {
            try {
                $order->sendNewOrderEmail();
                $this->_logger->debug(__METHOD__ . " (" . __LINE__ . ") - Sending a notification message to the customer!", null, 'shinesoftware_constriv.log');
            } catch (\Exception $e) {
                $this->_logger->debug(__METHOD__ . " (" . __LINE__ . ") - There was a problem when we try to send a notification message to the customer! " . $e->getMessage(), null, 'shinesoftware_constriv.log');
            }
            $this->setAdditionalInformation($order, Transaction::TYPE_AUTH, $additional_data);
            $this->_logger->debug(__METHOD__ . " (" . __LINE__ . ") - Saving additional information to the payment transaction: " . json_encode($additional_data), null, 'shinesoftware_constriv.log');
            if ($this->_configScopeConfigInterface->getValue('payment/constriv_hosted/autoinvoice', ScopeInterface::SCOPE_STORE)) {
                $this->_createInvoice($order);
            } else {
                $this->_logger->debug(__METHOD__ . " (" . __LINE__ . ") - Auto invoice option is disabled! No invoice has been created!", null, 'shinesoftware_constriv.log');
            }
            $order->setState(Order::STATE_PROCESSING, true, "Payment confirmed from CONSTRIV...", false);
            $order->save();
            return true;
        } else {
            $this->_logger->debug(__METHOD__ . " (" . __LINE__ . ") ---> The Order Id has been not found!", Null, 'shinesoftware_constriv.log');
        }
        return false;
    }

    protected function _createInvoice($order) {
        if (!$order->canInvoice()) {
            return false;
        }
        $invoice = $order->prepareInvoice();
        $invoice->register();
        if ($invoice->canCapture()) {
            $invoice->capture();
        }
        $invoice->save();
        $order->addRelatedObject($invoice);
        return $invoice;
    }

    public function validate() {
        parent::validate();
        return $this;
    }

}

;
