<?php

namespace ShineSoftware\Constriv\Model\Gateways;

use Magento\Framework\Api\AttributeValueFactory;
use Magento\Framework\Api\ExtensionAttributesFactory;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\DataObject;
use Magento\Framework\Data\Collection\AbstractDb;
use Magento\Framework\Locale\Resolver;
use Magento\Framework\Model\Context;
use Magento\Framework\Model\ResourceModel\AbstractResource;
use Magento\Framework\Registry;
use Magento\Framework\UrlInterface;
use Magento\Payment\Helper\Data as HelperData;
use Magento\Payment\Model\Method\Logger;
use Magento\Sales\Model\Order\Payment\Transaction;
use Magento\Store\Model\ScopeInterface;
//use PAYMENT\HOSTED\GATEWAY;
use ShineSoftware\Constriv\Model\Gateway;
use ShineSoftware\Constriv\Model\Source\CurrenciesFactory;

class Hosted extends Gateway {

    /**
     * @var CurrenciesFactory
     */
    protected $_sourceCurrenciesFactory;

    /**
     * @var UrlInterface
     */
    protected $_frameworkUrlInterface;

    /**
     * @var Resolver
     */
    protected $_localeResolver;

    public function __construct(Context $context, Registry $registry, ExtensionAttributesFactory $extensionFactory, AttributeValueFactory $customAttributeFactory, HelperData $paymentData, ScopeConfigInterface $scopeConfig, Logger $logger, CurrenciesFactory $sourceCurrenciesFactory, UrlInterface $frameworkUrlInterface, Resolver $localeResolver, AbstractResource $resource = null, AbstractDb $resourceCollection = null, array $data = []) {
        $this->_sourceCurrenciesFactory = $sourceCurrenciesFactory;
        $this->_frameworkUrlInterface = $frameworkUrlInterface;
        $this->_localeResolver = $localeResolver;

        parent::__construct($context, $registry, $extensionFactory, $customAttributeFactory, $paymentData, $scopeConfig, $logger, $resource, $resourceCollection, $data);
    }

    protected $_version = '2.0.0.1';
    protected $_code = 'constriv_hosted';
    protected $_infoBlockType = 'constriv/hosted_info';
    protected $_formBlockType = 'constriv/hosted_form';
    protected $_parameters = null;
    protected $_buffer = null;
    protected $_paymentId = null;
    protected $_paymentUrl = null;
    protected $_hasError = null;
    protected $_errorCode = null;
    protected $_errorMessage = null;
    protected $_isGateway = true;
    protected $_canAuthorize = true;
    protected $_canCapture = true;
    protected $_canRefund = true;
    protected $_canVoid = true;
    protected $_canUseInternal = false;
    protected $_canRefundInvoicePartial = true;
    protected $_canCapturePartial = true;

    public function getOrderPlaceRedirectUrl() {
        return Mage::getUrl('constriv/hosted/redirect');
    }

    public function init($order) {
        $currencies = $this->_sourceCurrenciesFactory->create();
        $version = Mage::getConfig()->getModuleConfig("ShineSoftware_Constriv")->version;
        if ($this->_scopeConfig->getValue('payment/constriv_moto_required_settings/debug', ScopeInterface::SCOPE_STORE)) {
            $mode = "Test Mode";
        } else {
            $mode = "Live Mode";
        }
        $orderId = $order->getIncrementId();
        $this->_logger->debug(__METHOD__ . " (" . __LINE__ . ") - Consorzio Triveneto $version Starts - Order: $orderId - Merchant Id (" . $this->_scopeConfig->getValue('payment/constriv_hosted_required_settings/id', ScopeInterface::SCOPE_STORE) . ") - $mode", Null, 'shinesoftware_constriv.log');
        $data = [
            'id' => $this->_scopeConfig->getValue('payment/constriv_hosted_required_settings/id', ScopeInterface::SCOPE_STORE),
            'password' => $this->_scopeConfig->getValue('payment/constriv_hosted_required_settings/password', ScopeInterface::SCOPE_STORE),
            'page' => 'MOBILE',
            'amt' => number_format($order->getGrandTotal(), 2, '.', ''),
            'currencycode' => $currencies->findCurrencyIsoCode($order->getOrderCurrencyCode()),
            'trackid' => $order->getIncrementId(),
            'langid' => $this->getLanguage(),
            'responseURL' => $this->_frameworkUrlInterface->getBaseUrl() . 'constriv/hosted/response',
            'errorURL' => $this->_frameworkUrlInterface->getBaseUrl() . 'constriv/hosted/error',
            'member' => $order->getCustomerFirstname() . " " . $order->getCustomerLastname(),
            'udf3' => "EMAILADDR:" . $order->getCustomerEmail(),
        ];
        if ($this->_scopeConfig->getValue('payment/constriv_hosted_base_settings/credit_card_sha', ScopeInterface::SCOPE_STORE)) {
            $data['udf1'] = "SHA1";
        }
        if ($this->_scopeConfig->getValue('payment/constriv_hosted_base_settings/hpp_timeout', ScopeInterface::SCOPE_STORE)) {
            $timeout = $this->_scopeConfig->getValue('payment/constriv_hosted_base_settings/hpp_timeout', ScopeInterface::SCOPE_STORE);
            if (is_numeric($timeout)) {
                $data['udf5'] = "HPP_TIMEOUT=$timeout";
            } else {
                $this->_logLoggerInterface->debug(__METHOD__ . " (" . __LINE__ . ") - Wrong Timeout setting! Please check the configuration panel and fix the error!", Null, 'shinesoftware_constriv.log');
            }
        }
        if ("authorize_capture" == $this->_scopeConfig->getValue('payment/constriv_hosted_required_settings/request_type', ScopeInterface::SCOPE_STORE)) {
            $data['action'] = 1;
        } else {
            $data['action'] = 4;
        }
        $url = $this->getEndpointUrl(self::GATEWAY, 'payment_init');
        $this->call($url, $data);
        if ($this->hasError()) {
            throw new \Exception($this->getErrorMessage());
        } else {
            $this->parseInitResponse();
            $order->getPayment()->setAdditionalInformation('paymentid', $this->getPaymentId());
            return $this->getHostedPageURL();
        }
    }

    public function getLanguage() {
        $default_locale = substr($this->_localeResolver->getLocale(), -2);
        switch ($default_locale) {
            case "IT":
                return "ITA";
                break;
            case "DE":
                return "DEU";
                break;
            case "EN":
                return "USA";
                break;
            case "US":
                return "USA";
                break;
            case "ES":
                return "SPA";
                break;
            case "FR":
                return "FRA";
                break;
            default:
                return "ITA";
                break;
        }
    }

    public function authorize(\Magento\Payment\Model\InfoInterface $payment, $amount) {
        if (!$this->canAuthorize()) {
            throw new \Exception(__('Authorize action is not available.'));
        }
        $payment->setTransactionId($payment->getOrder()->getIncrementId() . "-auth")
                ->setIsTransactionClosed(0)
                ->setTransactionAdditionalInfo(Transaction::RAW_DETAILS, []);
        return $this;
    }

    public function capture(\Magento\Payment\Model\InfoInterface $payment, $amount) {
        $currencies = $this->_sourceCurrenciesFactory->create();
        $info = $this->getInfoInstance();
        if (!$this->canCapture()) {
            throw new \Exception(__('Capture action is not available.'));
        }
        $parentTransaction = $info->lookupTransaction(false, Transaction::TYPE_AUTH);
        if ($parentTransaction) {
            $info->setParentTransactionId($parentTransaction->getTxnId());
            $addInfo = $parentTransaction->getAdditionalInformation(Transaction::RAW_DETAILS);
            if (empty($addInfo)) {
                throw new \Exception('No Authorization transaction information have been found!');
            }
        } else {
            throw new \Exception('No Authorization transaction has been found!');
        }
        if ("authorize_capture" == $this->_scopeConfig->getValue('payment/constriv_hosted_required_settings/request_type', ScopeInterface::SCOPE_STORE)) {
            $this->_logLoggerInterface->debug(__METHOD__ . " (" . __LINE__ . ") - Transaction payment has been already captured!", Null, 'shinesoftware_constriv.log');
            $payment->setStatus(self::STATUS_APPROVED)
                    ->setLastTransId($info->getLastTransId())
                    ->setTransactionId($addInfo['tranid'] . "-capture")
                    ->setIsTransactionClosed(1)
                    ->setTransactionAdditionalInfo(Transaction::RAW_DETAILS, $addInfo);
            return $this;
        }
        $data = [
            'id' => $this->_scopeConfig->getValue('payment/constriv_hosted_required_settings/id', ScopeInterface::SCOPE_STORE),
            'password' => $this->_scopeConfig->getValue('payment/constriv_hosted_required_settings/password', ScopeInterface::SCOPE_STORE),
            'action' => 5,
            'currencycode' => $currencies->findCurrencyIsoCode($payment->getOrder()->getOrderCurrencyCode()),
            'amt' => number_format($amount, 2, '.', ''),
            'paymentid' => $addInfo['paymentid'],
            'transid' => $addInfo['tranid'],
            'trackid' => $addInfo['trackid'],
        ];
        $url = $this->getEndpointUrl(self::GATEWAY, 'callback');
        $this->call($url, $data);
        $this->_logLoggerInterface->debug(__METHOD__ . " (" . __LINE__ . ") - URI: " . $url, Null, 'shinesoftware_constriv.log');
        $this->_logLoggerInterface->debug(__METHOD__ . " (" . __LINE__ . ") - Init parameters: " . json_encode($this->getParams()), Null, 'shinesoftware_constriv.log');
        if (!$this->hasError()) {
            $result = $this->getParams();
            $this->_logLoggerInterface->debug(__METHOD__ . " (" . __LINE__ . ") - Capture Payment Transaction: " . json_encode($this->getParams()), Null, 'shinesoftware_constriv.log');
            $payment->setStatus(self::STATUS_APPROVED)
                    ->setLastTransId($info->getLastTransId())
                    ->setTransactionId($addInfo['tranid'] . "-capture")
                    ->setIsTransactionClosed(1)
                    ->setTransactionAdditionalInfo(Transaction::RAW_DETAILS, $result);
        } else {
            $this->_logLoggerInterface->debug(__METHOD__ . " (" . __LINE__ . ") - Capture Payment Transaction: " . json_encode($this->getErrorMessage()), Null, 'shinesoftware_constriv.log');
            throw new \Exception($this->getErrorMessage());
        }
        return $this;
    }

    public function refund(\Magento\Payment\Model\InfoInterface $payment, $amount) {
        $currencies = $this->_sourceCurrenciesFactory->create();
        $info = $this->getInfoInstance();
        if (!$this->canRefund()) {
            throw new \Exception(__('Refund action is not available.'));
        }
        $parentTransaction = $info->lookupTransaction(false, Transaction::TYPE_CAPTURE);
        if ($parentTransaction) {
            $info->setParentTransactionId($parentTransaction->getTxnId());
            $addInfo = $parentTransaction->getAdditionalInformation(Transaction::RAW_DETAILS);
            if (empty($addInfo)) {
                throw new \Exception('No Authorization transaction information have been found!');
            }
        } else {
            throw new \Exception('No Authorization transaction has been found!');
        }
        $data = [
            'id' => $this->_scopeConfig->getValue('payment/constriv_hosted_required_settings/id', ScopeInterface::SCOPE_STORE),
            'password' => $this->_scopeConfig->getValue('payment/constriv_hosted_required_settings/password', ScopeInterface::SCOPE_STORE),
            'action' => 2,
            'currencycode' => $currencies->findCurrencyIsoCode($payment->getOrder()->getOrderCurrencyCode()),
            'amt' => number_format($amount, 2, '.', ''),
            'paymentid' => $addInfo['paymentid'],
            'transid' => $addInfo['tranid'],
            'trackid' => $addInfo['trackid'],
        ];
        $url = $this->getEndpointUrl(self::GATEWAY, 'callback');
        $this->call($url, $data);
        $this->_logLoggerInterface->debug(__METHOD__ . " (" . __LINE__ . ") - URI: " . $url, Null, 'shinesoftware_constriv.log');
        $this->_logLoggerInterface->debug(__METHOD__ . " (" . __LINE__ . ") - Init parameters: " . json_encode($this->getParams()), Null, 'shinesoftware_constriv.log');
        if (!$this->hasError()) {
            $result = $this->getParams();
            $this->_logLoggerInterface->debug(__METHOD__ . " (" . __LINE__ . ") - Refund Payment Transaction: " . json_encode($this->getParams()), Null, 'shinesoftware_constriv.log');
            $payment->setStatus(self::STATUS_APPROVED)
                    ->setLastTransId($info->getLastTransId())
                    ->setTransactionId($addInfo['tranid'] . "-refund")
                    ->setIsTransactionClosed(1)
                    ->setTransactionAdditionalInfo(Transaction::RAW_DETAILS, $result);
        } else {
            $this->_logLoggerInterface->debug(__METHOD__ . " (" . __LINE__ . ") - Refund Payment Transaction: " . json_encode($this->getErrorMessage()), Null, 'shinesoftware_constriv.log');
            throw new \Exception($this->getErrorMessage());
        }
        return $this;
    }

    public function void(\Magento\Payment\Model\InfoInterface $payment) {
        $currencies = $this->_sourceCurrenciesFactory->create();
        $info = $this->getInfoInstance();
        if (!$this->canVoid($payment)) {
            throw new \Exception(__('Void action is not available.'));
        }
        $parentTransaction = $info->lookupTransaction(false, Transaction::TYPE_AUTH);
        if ($parentTransaction) {
            $info->setParentTransactionId($parentTransaction->getTxnId());
            $addInfo = $parentTransaction->getAdditionalInformation(Transaction::RAW_DETAILS);
            if (empty($addInfo)) {
                throw new \Exception('No Authorization transaction information have been found!');
            }
        } else {
            throw new \Exception('No Authorization transaction has been found!');
        }
        $amount = $payment->getOrder()->getGrandTotal();
        $data = [
            'id' => $this->_scopeConfig->getValue('payment/constriv_hosted_required_settings/id', ScopeInterface::SCOPE_STORE),
            'password' => $this->_scopeConfig->getValue('payment/constriv_hosted_required_settings/password', ScopeInterface::SCOPE_STORE),
            'action' => 9,
            'currencycode' => $currencies->findCurrencyIsoCode($payment->getOrder()->getOrderCurrencyCode()),
            'amt' => number_format($amount, 2, '.', ''),
            'paymentid' => $addInfo['paymentid'],
            'transid' => $addInfo['tranid'],
            'trackid' => $addInfo['trackid'],
        ];
        $url = $this->getEndpointUrl(self::GATEWAY, 'callback');
        $this->call($url, $data);
        $this->_logLoggerInterface->debug(__METHOD__ . " (" . __LINE__ . ") - URI: " . $url, Null, 'shinesoftware_constriv.log');
        $this->_logLoggerInterface->debug(__METHOD__ . " (" . __LINE__ . ") - Init parameters: " . json_encode($this->getParams()), Null, 'shinesoftware_constriv.log');
        if (!$this->hasError()) {
            $result = $this->getParams();
            $this->_logLoggerInterface->debug(__METHOD__ . " (" . __LINE__ . ") - Void Payment Transaction: " . json_encode($this->getParams()), Null, 'shinesoftware_constriv.log');
            $payment->setStatus(self::STATUS_APPROVED)
                    ->setLastTransId($info->getLastTransId())
                    ->setTransactionId($addInfo['tranid'] . "-void")
                    ->setIsTransactionClosed(1)
                    ->setTransactionAdditionalInfo(Transaction::RAW_DETAILS, $result);
        } else {
            $this->_logLoggerInterface->debug(__METHOD__ . " (" . __LINE__ . ") - Void Payment Transaction: " . json_encode($this->getErrorMessage()), Null, 'shinesoftware_constriv.log');
            throw new \Exception($this->getErrorMessage());
        }
        return $this;
    }

    public function getTitle() {
        return $this->_scopeConfig->getValue('payment/constriv_hosted_required_settings/title', ScopeInterface::SCOPE_STORE);
    }

    public function isAvailable(\Magento\Quote\Api\Data\CartInterface $quote = null) {
        $id = $this->_scopeConfig->getValue('payment/constriv_hosted_required_settings/id', ScopeInterface::SCOPE_STORE);
        $password = $this->_scopeConfig->getValue('payment/constriv_hosted_required_settings/password', ScopeInterface::SCOPE_STORE);
        $active = $this->_scopeConfig->getValue('payment/constriv_hosted/active', ScopeInterface::SCOPE_STORE);
        return parent::isAvailable($quote) && $active && trim($id) != '' && trim($password) != '';
    }

    public function assignData(\Magento\Framework\DataObject $data) {
        $info = $this->getInfoInstance();
        $info->setAdditionalInformation('savetoken', $data->getCcSavetoken());
        return $this;
    }

    private function getEndpointUrl($gateway = 'hosted', $type = 'payment_init') {
        if ($this->_scopeConfig->getValue("payment/constriv_$gateway" . "_required_settings/debug", ScopeInterface::SCOPE_STORE)) {
            return $this->_scopeConfig->getValue("payment/constriv_$gateway/uri/test/$type", ScopeInterface::SCOPE_STORE);
        } else {
            return $this->_scopeConfig->getValue("payment/constriv_$gateway/uri/live/$type", ScopeInterface::SCOPE_STORE);
        }
    }

    public function getHostedPageURL() {
        return $this->_paymentUrl . "?PaymentID=" . $this->_paymentId;
    }

    public function parseInitResponse() {
        list($this->_paymentId, $this->_paymentUrl) = explode(":", $this->_buffer, 2);
    }

    public function parsePaymentResponse() {
        list($result, $auth, $ref, $avr, $date, $transid, $trackid, $udf1, $udf2, $udf3, $udf4, $udf5) = explode(":", $this->_buffer);
        die($this->_buffer);
    }

    public function hasError() {
        if ($this->_hasError) {
            $this->_logLoggerInterface->debug(__METHOD__ . " (" . __LINE__ . ") ---- Error: " . $this->getErrorMessage(), Null, 'shinesoftware_constriv.log');
        } else {
            return false;
        }
        return true;
    }

    public function getErrorCode() {
        return $this->_errorCode;
    }

    public function getErrorMessage() {
        return $this->_errorMessage;
    }

    public function getParams() {
        return $this->_parameters;
    }

    public function getPaymentId() {
        return $this->_paymentId;
    }

    public function call($URI, $parameters, $tls = true) {
        try {
            $version = curl_version();
            $curl = curl_init();
            curl_setopt($curl, CURLOPT_URL, $URI);
            curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
            curl_setopt($curl, CURLOPT_POSTFIELDS, http_build_query($parameters));
            curl_setopt($curl, CURLOPT_FAILONERROR, 1);
            curl_setopt($curl, CURLOPT_HEADER, 0);
            $this->_buffer = curl_exec($curl);
            if (curl_error($curl)) {
                $this->_logLoggerInterface->debug(__METHOD__ . " (" . __LINE__ . ") - cURL v. " . $version['version'] . " error: " . curl_error($curl), Null, 'shinesoftware_constriv.log');
                $this->_hasError = true;
                $this->_errorCode = null;
                $this->_errorMessage = curl_error($curl);
                curl_close($curl);
                return false;
            }
            curl_close($curl);
            if (substr($this->_buffer, 0, 7) == '!ERROR!') {
                $this->_hasError = true;
                $this->_errorCode = $this->_buffer;
                $this->_errorMessage = $this->_buffer;
            } elseif (empty($this->_buffer)) {
                $this->_logLoggerInterface->debug(__METHOD__ . " (" . __LINE__ . ") - Buffer empty! Please check your cURL module in your php configuration!", Null, 'shinesoftware_constriv.log');
                return false;
            }
            $this->_logLoggerInterface->debug(__METHOD__ . " (" . __LINE__ . ") - URI: " . $URI, Null, 'shinesoftware_constriv.log');
            $this->_logLoggerInterface->debug(__METHOD__ . " (" . __LINE__ . ") - Init parameters: " . json_encode($parameters), Null, 'shinesoftware_constriv.log');
            $this->_logLoggerInterface->debug(__METHOD__ . " (" . __LINE__ . ") - Response from the bank: " . $this->_buffer, Null, 'shinesoftware_constriv.log');
            $this->_parameters = $parameters;
        } catch (\Exception $e) {
            $this->_logLoggerInterface->debug(__METHOD__ . " (" . __LINE__ . ") - Error: " . $e->getMessage(), Null, 'shinesoftware_constriv.log');
            return false;
        }
        return true;
    }

    public function getIssuerError() {
        $errorcode = $this->_errorCode;
        $codes = [
            '01' => 'Refer to card issuer',
            '01' => 'ERROR',
            '02' => 'Refer to special conditions for card issuer',
            '03' => 'Invalid merchant',
            '04' => 'Pick-up card',
            '05' => 'Do not honor',
            '06' => 'Error',
            '07' => 'Pick-up card, special condition',
            '08' => 'Honor with identification',
            '09' => 'Request in progress',
            '11' => 'Approved (VIP)',
            '12' => 'Invalid transaction',
            '13' => 'Invalid amount',
            '14' => 'Invalid card number (no such number)',
            '15' => 'No such issuer',
            '30' => 'Format error',
            '31' => 'Bank not supported by switch',
            '33' => 'Expired card',
            '34' => 'Suspected fraud',
            '35' => 'Card acceptor contact acquirer',
            '36' => 'Restricted card',
            '37' => 'Card acceptor call acquirer security',
            '38' => 'Allowable PIN tries exceeded',
            '39' => 'No credit account',
            '41' => 'Lost card',
            '43' => 'Stolen card, pick-up',
            '51' => 'Not sufficient funds',
            '54' => 'Expired card',
            '55' => 'Incorrect personal identification number',
            '56' => 'No card record',
            '57' => 'Transaction not permitted to cardholder',
            '58' => 'Transaction not permitted to terminal',
            '61' => 'Exceeds withdrawal amount limit',
            '62' => 'Restricted card',
            '65' => 'Exceeds withdrawal frequency limit',
            '68' => 'Response received too late',
            '75' => 'Allowable number of PIN tries exceeded',
            '90' => 'Cut-off is in process, a switch is ending business for a day and starting the next (transaction can be sent again in a few minutes)',
            '91' => 'Issuer or switch is inoperative',
            '92' => 'Financial institution or intermediate network facility cannot be found for routing',
            '94' => 'Duplicate transmission',
            '96' => 'System malfunction',
            'S4' => 'PTLF full'
        ];
        if (in_array($errorcode, array_keys($codes))) {
            return $codes[$errorcode];
        }
        return "Issuer Error Code: $errorcode!";
    }

    public function getVersion() {
        return $this->_version;
    }

}

;
