<?php

namespace ShineSoftware\Constriv\Model\Gateways;

use CURLOPT\SSL\CIPHER\LISTCIPHER;
use Magento\Framework\Api\AttributeValueFactory;
use Magento\Framework\Api\ExtensionAttributesFactory;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\DataObject;
use Magento\Framework\Data\Collection\AbstractDb;
use Magento\Framework\Locale\Resolver;
use Magento\Framework\Model\Context;
use Magento\Framework\Model\ResourceModel\AbstractResource;
use Magento\Framework\Module\ModuleListInterface;
use Magento\Framework\Registry;
use Magento\Framework\Stdlib\DateTime\TimezoneInterface;
use Magento\Payment\Helper\Data as HelperData;
use Magento\Payment\Model\Method\Cc;
use Magento\Payment\Model\Method\Logger;
use Magento\Quote\Model\Quote\Payment;
use Magento\Sales\Model\OrderFactory;
use Magento\Sales\Model\Order\Payment as OrderPayment;
use Magento\Sales\Model\Order\Payment\Transaction;
use Magento\Store\Model\ScopeInterface;
use ShineSoftware\Constriv\Model\Source\CurrenciesFactory;

class Moto extends Cc {

    /**
     * @var CurrenciesFactory
     */
    protected $_sourceCurrenciesFactory;

    /**
     * @var OrderFactory
     */
    protected $_modelOrderFactory;

    /**
     * @var Resolver
     */
    protected $_localeResolver;

    public function __construct(Context $context, Registry $registry, ExtensionAttributesFactory $extensionFactory, AttributeValueFactory $customAttributeFactory, HelperData $paymentData, ScopeConfigInterface $scopeConfig, Logger $logger, ModuleListInterface $moduleList, TimezoneInterface $localeDate, CurrenciesFactory $sourceCurrenciesFactory, OrderFactory $modelOrderFactory, Resolver $localeResolver, AbstractResource $resource = null, AbstractDb $resourceCollection = null, array $data = []) {
        $this->_sourceCurrenciesFactory = $sourceCurrenciesFactory;
        $this->_modelOrderFactory = $modelOrderFactory;
        $this->_localeResolver = $localeResolver;

        parent::__construct($context, $registry, $extensionFactory, $customAttributeFactory, $paymentData, $scopeConfig, $logger, $moduleList, $localeDate, $resource, $resourceCollection, $data);
    }

    protected $_version = '2.0.0.0';
    protected $_code = 'constriv_moto';
    protected $_infoBlockType = 'constriv/moto_info';
    protected $_formBlockType = 'constriv/moto_form';
    protected $xml;
    protected $parameters;
    protected $result;
    protected $_isGateway = true;
    protected $_canUseCheckout = true;
    protected $_canAuthorize = true;
    protected $_canCapture = true;
    protected $_canCapturePartial = true;
    protected $_canRefund = true;
    protected $_canRefundInvoicePartial = true;
    protected $_canVoid = true;
    protected $_canUseInternal = true;

    public function authorize(\Magento\Payment\Model\InfoInterface $payment, $amount) {
        if (!$this->canAuthorize()) {
            throw new \Exception(__('Authorize action is not available.'));
        }
        $version = Mage::getConfig()->getModuleConfig("ShineSoftware_Constriv")->version;
        if ($this->_scopeConfig->getValue('payment/constriv_moto/debug', ScopeInterface::SCOPE_STORE)) {
            $mode = "Test Mode";
        } else {
            $mode = "Live Mode";
        }
        $orderId = $this->_getOrderId();
        $this->_logger->debug(__METHOD__ . " (" . __LINE__ . ") - Consorzio Triveneto $version Starts - Order: $orderId - Merchant Id (" . $this->_scopeConfig->getValue('payment/constriv_moto/id', ScopeInterface::SCOPE_STORE) . ") - $mode", Null, 'shinesoftware_constriv.log');
        $currencies = $this->_sourceCurrenciesFactory->create();
        $data = [
            'id' => $this->_scopeConfig->getValue('payment/constriv_moto/id', ScopeInterface::SCOPE_STORE),
            'password' => $this->_scopeConfig->getValue('payment/constriv_moto/password', ScopeInterface::SCOPE_STORE),
            'action' => '4',
            'amt' => $this->_getAmount(),
            'currencycode' => $currencies->findCurrencyIsoCode($this->_getCurrency()),
            'trackid' => $orderId,
            'card' => $this->getInfoInstance()->getCcNumber(),
            'cvv2' => $this->getInfoInstance()->getCcCid(),
            'expmonth' => $this->getInfoInstance()->getCcExpMonth(),
            'expyear' => $this->getInfoInstance()->getCcExpYear(),
            'member' => $payment->getOrder()->getCustomerFirstname() . " " . $payment->getOrder()->getCustomerLastname(),
            'addr' => $payment->getOrder()->getBillingAddress()->getFormated(),
        ];
        $this->call($this->getEndpointUrl('moto', 'vet'), $data);
        if ($this->hasError()) {
            throw new \Exception($this->getErrorCode() . " " . $this->getErrorMessage());
        }
        if ($this->hasIssuerError()) {
            if (!$this->isEnrolled()) {
                throw new \Exception($this->getIssuerError());
            }
        }
        $result = $this->asArray();
        $payment->setTransactionId($result['trackid'] . "-auth")
                ->setIsTransactionClosed(0)
                ->setTransactionAdditionalInfo(Transaction::RAW_DETAILS, $result);
        return $this;
    }

    public function capture(\Magento\Payment\Model\InfoInterface $payment, $amount) {
        if (!$this->canCapture()) {
            throw new \Exception(__('Capture action is not available.'));
        }
        $info = $this->getInfoInstance();
        $parentTransaction = $info->lookupTransaction(false, Transaction::TYPE_AUTH);
        if ($parentTransaction) {
            $info->setParentTransactionId($parentTransaction->getTxnId());
            $addInfo = $parentTransaction->getAdditionalInformation(Transaction::RAW_DETAILS);
            if (empty($addInfo)) {
                throw new \Exception('No Authorization transaction information have been found!');
            }
        } else {
            throw new \Exception('No Authorization transaction has been found!');
        }
        $data = [
            'ID' => $this->_scopeConfig->getValue('payment/constriv_moto/id', ScopeInterface::SCOPE_STORE),
            'password' => $this->_scopeConfig->getValue('payment/constriv_moto/password', ScopeInterface::SCOPE_STORE),
            'action' => '5',
            'paymentid' => $addInfo['paymentid'],
            'tranid' => $addInfo['tranid'],
            'trackid' => $addInfo['trackid'],
            'amt' => number_format($amount, 2),
        ];
        $this->call($this->getEndpointUrl('moto', 'trans'), $data);
        $this->_logger->debug(__METHOD__ . " (" . __LINE__ . ") ---> URI: " . $this->getEndpointUrl('moto'), Null, 'shinesoftware_constriv.log');
        $this->_logger->debug(__METHOD__ . " (" . __LINE__ . ") ---> Init parameters: " . json_encode($this->getParams()), Null, 'shinesoftware_constriv.log');
        if (!$this->hasError()) {
            $result = $this->asArray();
            $this->_logger->debug(__METHOD__ . " (" . __LINE__ . ") ---> Capture Payment Transaction: " . json_encode($this->asArray()), Null, 'shinesoftware_constriv.log');
            $payment->setStatus(self::STATUS_APPROVED)
                    ->setLastTransId($info->getLastTransId())
                    ->setTransactionId($addInfo['tranid'] . "-capture")
                    ->setIsTransactionClosed(1)
                    ->setTransactionAdditionalInfo(Transaction::RAW_DETAILS, $result);
        } else {
            $this->_logger->debug(__METHOD__ . " (" . __LINE__ . ") ---> Capture Payment Transaction: " . json_encode($this->getErrorMessage()), Null, 'shinesoftware_constriv.log');
            throw new \Exception($this->getErrorMessage());
        }
        return $this;
    }

    public function refund(\Magento\Payment\Model\InfoInterface $payment, $amount) {
        if (!$this->canRefund()) {
            throw new \Exception(__('Refund action is not available.'));
        }
        $info = $this->getInfoInstance();
        $parentTransaction = $info->lookupTransaction(false, Transaction::TYPE_AUTH);
        if ($parentTransaction) {
            $info->setParentTransactionId($parentTransaction->getTxnId());
            $addInfo = $parentTransaction->getAdditionalInformation(Transaction::RAW_DETAILS);
            if (empty($addInfo)) {
                throw new \Exception('No Authorization transaction information have been found!');
            }
        } else {
            throw new \Exception('No Authorization transaction has been found!');
        }
        $data = [
            'ID' => $this->_scopeConfig->getValue('payment/constriv_moto/id', ScopeInterface::SCOPE_STORE),
            'password' => $this->_scopeConfig->getValue('payment/constriv_moto/password', ScopeInterface::SCOPE_STORE),
            'action' => '2',
            'paymentid' => $addInfo['paymentid'],
            'tranid' => $addInfo['tranid'],
            'trackid' => $addInfo['trackid'],
            'amt' => number_format($amount, 2),
        ];
        $this->call($this->getEndpointUrl('moto', 'trans'), $data);
        $this->_logger->debug(__METHOD__ . " (" . __LINE__ . ") ---> URI: " . $this->getEndpointUrl('moto'), Null, 'shinesoftware_constriv.log');
        $this->_logger->debug(__METHOD__ . " (" . __LINE__ . ") ---> Init parameters: " . json_encode($this->getParams()), Null, 'shinesoftware_constriv.log');
        if (!$this->hasError()) {
            $result = $this->asArray();
            $this->_logger->debug(__METHOD__ . " (" . __LINE__ . ") ---> Refund Payment Transaction: " . json_encode($this->asArray()), Null, 'shinesoftware_constriv.log');
            $payment->setStatus(self::STATUS_APPROVED)
                    ->setLastTransId($info->getLastTransId())
                    ->setTransactionId($addInfo['tranid'] . "-refund")
                    ->setIsTransactionClosed(1)
                    ->setTransactionAdditionalInfo(Transaction::RAW_DETAILS, $result);
        } else {
            $this->_logger->debug(__METHOD__ . " (" . __LINE__ . ") ---> Refund Payment Transaction: " . json_encode($this->getErrorMessage()), Null, 'shinesoftware_constriv.log');
            throw new \Exception($this->getErrorMessage());
        }
        return $this;
    }

    public function void(\Magento\Payment\Model\InfoInterface $payment) {
        if (!$this->canVoid($payment)) {
            throw new \Exception(__('Void action is not available.'));
        }
        $info = $this->getInfoInstance();
        $amount = $info->getData('base_amount_authorized');
        $parentTransaction = $info->lookupTransaction(false, Transaction::TYPE_AUTH);
        if ($parentTransaction) {
            $info->setParentTransactionId($parentTransaction->getTxnId());
            $addInfo = $parentTransaction->getAdditionalInformation(Transaction::RAW_DETAILS);
            if (empty($addInfo)) {
                throw new \Exception('No Authorization transaction information have been found!');
            }
        } else {
            throw new \Exception('No Authorization transaction has been found!');
        }
        $data = [
            'ID' => $this->_scopeConfig->getValue('payment/constriv_moto/id', ScopeInterface::SCOPE_STORE),
            'password' => $this->_scopeConfig->getValue('payment/constriv_moto/password', ScopeInterface::SCOPE_STORE),
            'action' => '9',
            'paymentid' => $addInfo['paymentid'],
            'tranid' => $addInfo['tranid'],
            'trackid' => $addInfo['trackid'],
            'amt' => number_format($amount, 2),
        ];
        $this->call($this->getEndpointUrl('moto', 'trans'), $data);
        $this->_logger->debug(__METHOD__ . " (" . __LINE__ . ") ---> URI: " . $this->getEndpointUrl('moto'), Null, 'shinesoftware_constriv.log');
        $this->_logger->debug(__METHOD__ . " (" . __LINE__ . ") ---> Init parameters: " . json_encode($this->getParams()), Null, 'shinesoftware_constriv.log');
        if (!$this->hasError()) {
            $result = $this->asArray();
            $this->_logger->debug(__METHOD__ . " (" . __LINE__ . ") ---> Void Payment Transaction: " . json_encode($this->asArray()), Null, 'shinesoftware_constriv.log');
            $payment->setStatus(self::STATUS_APPROVED)
                    ->setLastTransId($info->getLastTransId())
                    ->setTransactionId($addInfo['tranid'] . "-void")
                    ->setIsTransactionClosed(1)
                    ->setTransactionAdditionalInfo(Transaction::RAW_DETAILS, $result);
        } else {
            $this->_logger->debug(__METHOD__ . " (" . __LINE__ . ") ---> Void Payment Transaction: " . json_encode($this->getErrorMessage()), Null, 'shinesoftware_constriv.log');
            throw new \Exception($this->getErrorMessage());
        }
        return $this;
    }

    public function requestPatTransaction($orderId, $PaRes) {
        $order = $this->_modelOrderFactory->create();
        $order->loadByIncrementId($orderId);
        $payment = $order->getPayment();
        $transaction = $payment->lookupTransaction(false, Transaction::TYPE_AUTH);
        if ($transaction) {
            $addInfo = $transaction->getAdditionalInformation(Transaction::RAW_DETAILS);
        }
        $data = [
            'id' => $this->_scopeConfig->getValue('payment/constriv_moto/id', ScopeInterface::SCOPE_STORE),
            'password' => $this->_scopeConfig->getValue('payment/constriv_moto/password', ScopeInterface::SCOPE_STORE),
            'paymentid' => $addInfo['paymentid'],
            'PARes' => $PaRes,
        ];
        $this->call($this->getEndpointUrl('moto', 'pat'), $data);
        if ($this->hasError()) {
            throw new \Exception($this->getErrorCode() . " " . $this->getErrorMessage());
        }
        $transaction->setAdditionalInformation(Transaction::RAW_DETAILS, $this->asArray());
        $transaction->save();
        return true;
    }

    public function getTitle() {
        return $this->_scopeConfig->getValue('payment/constriv_moto/title', ScopeInterface::SCOPE_STORE);
    }

    public function isAvailable(\Magento\Quote\Api\Data\CartInterface $quote = null) {
        $id = $this->_scopeConfig->getValue('payment/constriv_moto/id', ScopeInterface::SCOPE_STORE);
        $password = $this->_scopeConfig->getValue('payment/constriv_moto/password', ScopeInterface::SCOPE_STORE);
        $active = $this->_scopeConfig->getValue('payment/constriv_moto/active', ScopeInterface::SCOPE_STORE);
        return parent::isAvailable($quote) && $active && trim($id) != '' && trim($password) != '';
    }

    private function _isPlaceOrder() {
        $info = $this->getInfoInstance();
        if ($info instanceof Payment) {
            return false;
        } elseif ($info instanceof OrderPayment) {
            return true;
        }
    }

    private function _getCurrency() {
        $info = $this->getInfoInstance();
        if ($this->_isPlaceOrder()) {
            return $info->getOrder()->getBaseCurrencyCode();
        } else {
            return $info->getQuote()->getQuoteBaseCurrencyCode();
        }
    }

    private function _getOrderId() {
        $info = $this->getInfoInstance();
        if ($this->_isPlaceOrder()) {
            return $info->getOrder()->getIncrementId();
        } else {
            if (!$info->getQuote()->getReservedOrderId()) {
                $info->getQuote()->reserveOrderId();
            }
            return $info->getQuote()->getReservedOrderId();
        }
    }

    private function _getAmount() {
        $info = $this->getInfoInstance();
        if ($this->_isPlaceOrder()) {
            return (float) $info->getOrder()->getQuoteBaseGrandTotal();
        } else {
            return (float) $info->getQuote()->getBaseGrandTotal();
        }
    }

    public function assignData(\Magento\Framework\DataObject $data) {
        if (!($data instanceof DataObject)) {
            $data = new DataObject($data);
        }
        $info = $this->getInfoInstance();
        $info->setCcType($data->getCcType())
                ->setCcOwner($data->getCcOwner())
                ->setCcLast4(substr($data->getCcNumber(), -4))
                ->setCcNumber($data->getCcNumber())
                ->setCcCid($data->getCcCid())
                ->setCcExpMonth($data->getCcExpMonth())
                ->setCcExpYear($data->getCcExpYear())
                ->setCcSsIssue($data->getCcSsIssue())
                ->setCcSsStartMonth($data->getCcSsStartMonth())
                ->setCcSsStartYear($data->getCcSsStartYear());
        return $this;
    }

    public function getConfigData($field, $storeId = null) {
        if (null === $storeId) {
            $storeId = $this->getStore();
        }
        if ($field == "cctypes") {
            $subpath = "constriv_moto";
        } else {
            $subpath = $this->getCode();
        }
        $path = 'payment/' . $subpath . '/' . $field;
        return $this->_scopeConfig->getValue($path, ScopeInterface::SCOPE_STORE, $storeId);
    }

    public function getOrderPlaceRedirectUrl() {
        $info = $this->getInfoInstance();
        $data = $info->getQuote()->getAdditionalData();
        if (!empty($data['url'])) {
            $this->_logger->debug(__METHOD__ . " (" . __LINE__ . ") -----> Payment needs authorization by 3DSecure", Null, 'shinesoftware_constriv.log');
            return Mage::getUrl('constriv/moto/enrolled');
        }
        return false;
    }

    public function call($URI, $parameters, $tls = true) {
        $xmlString = $this->array_to_xml($parameters);
        try {
            $ch = curl_init($URI);
            curl_setopt($ch, CURLOPT_POST, 1);
            curl_setopt($ch, CURLOPT_POSTFIELDS, $xmlString);
            curl_setopt($ch, CURLOPT_HEADER, 0);
            if ($tls) {
                curl_setopt($ch, LISTCIPHER, 'TLSv1');
            }
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
            $buffer = curl_exec($ch);
            curl_close($ch);
            $this->xml = $this->parserResponse($buffer);
            $parameters = $this->hideCreditCardInfo($parameters);
            $this->_logger->debug(__METHOD__ . " (" . __LINE__ . ") ---> URI: " . $URI, Null, 'shinesoftware_constriv.log');
            $this->_logger->debug(__METHOD__ . " (" . __LINE__ . ") ---> Init parameters: " . json_encode($parameters), Null, 'shinesoftware_constriv.log');
            $this->_logger->debug(__METHOD__ . " (" . __LINE__ . ") ---> Response parameters: " . json_encode($this->asArray()), Null, 'shinesoftware_constriv.log');
            $this->parameters = $parameters;
        } catch (SoapFault $e) {
            return false;
        }
        return true;
    }

    public function isEnrolled() {
        if ((string) $this->xml->result == 'ENROLLED') {
            return true;
        }
        return false;
    }

    public function getEnrolledUrl() {
        if (!empty($this->xml->url)) {
            return (string) $this->xml->url;
        }
        return false;
    }

    private function hideCreditCardInfo($data) {
        if (!empty($data['card'])) {
            $data['card'] = sprintf('xxxx-xxxx-xxxx-%s', substr($data['card'], 12, strlen($data['card'])));
        }
        if (!empty($data['cvv2'])) {
            $data['cvv2'] = sprintf('xx%s', substr($data['cvv2'], 2, strlen($data['cvv2'])));
        }
        return $data;
    }

    private function array_to_xml($parameters) {
        $dom = new DomDocument();
        $xmldata = null;
        foreach ($parameters as $key => $val) {
            $child = $dom->createElement($key);
            $child->appendChild(new DomText($val));
            $xmldata .= $dom->saveXml($child);
        }
        return $xmldata;
    }

    private function parserResponse($xml) {
        return simplexml_load_string($xml);
    }

    public function asArray() {
        $json = json_encode($this->xml);
        $data = json_decode($json, true);
        unset($data['@attributes']);
        return $data;
    }

    public function hasError() {
        if (!empty($this->xml) && !empty($this->xml->error_code_tag)) {
            $this->_logger->debug(__METHOD__ . " (" . __LINE__ . ") ------> Error: " . $this->getErrorMessage(), Null, 'shinesoftware_constriv.log');
        } else {
            return false;
        }
        return true;
    }

    public function hasIssuerError() {
        if ((string) $this->xml->responsecode == "00") {
            $this->_logger->debug(__METHOD__ . " (" . __LINE__ . ") ------> Credit card issuer has approved the transaction!", Null, 'shinesoftware_constriv.log');
        } else {
            $this->_logger->debug(__METHOD__ . " (" . __LINE__ . ") ------> Credit card issuer has replied with code: " . (string) $this->xml->responsecode, Null, 'shinesoftware_constriv.log');
            return true;
        }
        return false;
    }

    public function getIssuerError() {
        $errorcode = (string) $this->xml->responsecode;
        $codes = [
            '01' => 'Refer to card issuer',
            '01' => 'ERROR',
            '02' => 'Refer to special conditions for card issuer',
            '03' => 'Invalid merchant',
            '04' => 'Pick-up card',
            '05' => 'Do not honor',
            '06' => 'Error',
            '07' => 'Pick-up card, special condition',
            '08' => 'Honor with identification',
            '09' => 'Request in progress',
            '11' => 'Approved (VIP)',
            '12' => 'Invalid transaction',
            '13' => 'Invalid amount',
            '14' => 'Invalid card number (no such number)',
            '15' => 'No such issuer',
            '30' => 'Format error',
            '31' => 'Bank not supported by switch',
            '33' => 'Expired card',
            '34' => 'Suspected fraud',
            '35' => 'Card acceptor contact acquirer',
            '36' => 'Restricted card',
            '37' => 'Card acceptor call acquirer security',
            '38' => 'Allowable PIN tries exceeded',
            '39' => 'No credit account',
            '41' => 'Lost card',
            '43' => 'Stolen card, pick-up',
            '51' => 'Not sufficient funds',
            '54' => 'Expired card',
            '55' => 'Incorrect personal identification number',
            '56' => 'No card record',
            '57' => 'Transaction not permitted to cardholder',
            '58' => 'Transaction not permitted to terminal',
            '61' => 'Exceeds withdrawal amount limit',
            '62' => 'Restricted card',
            '65' => 'Exceeds withdrawal frequency limit',
            '68' => 'Response received too late',
            '75' => 'Allowable number of PIN tries exceeded',
            '90' => 'Cut-off is in process, a switch is ending business for a day and starting the next (transaction can be sent again in a few minutes)',
            '91' => 'Issuer or switch is inoperative',
            '92' => 'Financial institution or intermediate network facility cannot be found for routing',
            '94' => 'Duplicate transmission',
            '96' => 'System malfunction',
            'S4' => 'PTLF full'
        ];
        if (in_array($errorcode, array_keys($codes))) {
            return $codes[$errorcode];
        }
        return "Issuer Error Code: $errorcode!";
    }

    public function getParams() {
        return $this->parameters;
    }

    public function getXml() {
        return $this->xml;
    }

    public function getErrorCode() {
        return (string) $this->xml->error_code_tag;
    }

    public function getErrorMessage() {
        return (string) $this->xml->error_text;
    }

    public function getErrorServiceTagMessage() {
        return (string) $this->xml->error_service_tag;
    }

    public function getPaymentId() {
        return (string) $this->xml->paymentid;
    }

    public function getHostedPageURL() {
        return (string) $this->xml->hostedpageurl;
    }

    public function getToken() {
        return (string) $this->xml->securitytoken;
    }

    public function getLanguage() {
        $default_locale = substr($this->_localeResolver->getLocale(), -2);
        switch ($default_locale) {
            case "IT":
                return "ITA";
                break;
            case "DE":
                return "DEU";
                break;
            case "EN":
                return "USA";
                break;
            case "US":
                return "USA";
                break;
            case "ES":
                return "SPA";
                break;
            case "FR":
                return "FRA";
                break;
            default:
                return "ITA";
                break;
        }
    }

    public function getEndpointUrl($gateway = 'hosted', $type = 'vet') {
        if ($this->_scopeConfig->getValue("payment/constriv_$gateway" . "_required_settings/debug", ScopeInterface::SCOPE_STORE)) {
            return $this->_scopeConfig->getValue("payment/constriv_$gateway/uri/test/$type", ScopeInterface::SCOPE_STORE);
        } else {
            return $this->_scopeConfig->getValue("payment/constriv_$gateway/uri/live/$type", ScopeInterface::SCOPE_STORE);
        }
    }

    function createToken($len = 8) {
        if (($len % 2) !== 0) {
            $len = 8;
        }
        $length = $len - 2;
        $conso = ['b', 'c', 'd', 'f', 'g', 'h', 'j', 'k', 'l', 'm', 'n', 'p', 'r', 's', 't', 'v', 'w', 'x', 'y', 'z'];
        $vocal = ['a', 'e', 'i', 'o', 'u'];
        $password = '';
        srand((double) microtime() * 1000000);
        $max = $length / 2;
        for ($i = 1; $i <= $max; $i++) {
            $password .= $conso[rand(0, 19)];
            $password .= $vocal[rand(0, 4)];
        }
        $password .= rand(10, 99);
        $newpass = $password;
        return $newpass;
    }

    public function getVersion() {
        return $this->_version;
    }

    public function hasVerification() {
        $configData = $this->_scopeConfig->getValue('payment/constriv_moto/useccv', ScopeInterface::SCOPE_STORE);
        if (is_null($configData)) {
            return true;
        }
        return (bool) $configData;
    }

}

;
